@@ -33,6 +33,7 @@ import numbers
 from mappyfile.tokens import (
     COMPOSITE_NAMES,
     SINGLETON_COMPOSITE_NAMES,
+    CONFIG_COMPOSITE_NAMES,
     REPEATED_KEYS,
     COMPLEX_TYPES,
     OBJECT_LIST_KEYS,
@@ -150,12 +151,18 @@ class PrettyPrinter:
     ) -> str:
         if (aligned_max_indent is None) or (aligned_max_indent == 0):
             aligned_max_indent = len(key) + 1
-        indent = " " * (aligned_max_indent - len(key))
+        if value == "":
+            indent = ""  # for keywords with no values, do not add a space
+        else:
+            indent = " " * (aligned_max_indent - len(key))
+
         tmpl = "{spacer}{key}{indent}{value}"
         d = {"spacer": spacer, "key": key, "value": value, "indent": indent}
         return tmpl.format(**d)
 
-    def process_key_dict(self, key: str, d: dict, level: int) -> list[str]:
+    def process_key_dict(
+        self, key: str, d: dict, level: int, quote_key: bool = True
+    ) -> list[str]:
         """
         Process key value dicts e.g. METADATA "key" "value"
         """
@@ -166,12 +173,14 @@ class PrettyPrinter:
         self._add_type_comment(level, comments, lines)
 
         lines += [self.add_start_line(key, level)]
-        lines += self.process_dict(d, level, comments)
+        lines += self.process_dict(d, level, comments, quote_key)
         lines.append(self.add_end_line(level, 1, key))
 
         return lines
 
-    def process_dict(self, d: dict, level: int, comments: dict) -> list[str]:
+    def process_dict(
+        self, d: dict, level: int, comments: dict, quote_key: bool = True
+    ) -> list[str]:
         """
         Process keys and values within a block
         """
@@ -184,7 +193,10 @@ class PrettyPrinter:
 
         for k, v in d.items():
             if not self.__is_metadata(k):
-                qk = self.quoter.add_quotes(k)
+                if quote_key:
+                    qk = self.quoter.add_quotes(k)
+                else:
+                    qk = k
                 qv = self.quoter.add_quotes(v)
                 line = self.__format_line(
                     self.whitespace(level, 2), qk, qv, aligned_max_indent
@@ -224,7 +236,7 @@ class PrettyPrinter:
         return lines
 
     def process_projection(
-        self, key, lst: (str | list[str]), level: int, projection_comments: str
+        self, key, lst: str | list[str], level: int, projection_comments: str
     ) -> list[str]:
         lines = [self.add_start_line(key, level)]
 
@@ -308,7 +320,7 @@ class PrettyPrinter:
             return True
         return False
 
-    def pprint(self, composites: (dict | list[dict])) -> str:
+    def pprint(self, composites: dict | list[dict]) -> str:
         """
         Print out a nicely indented Mapfile
         """
@@ -323,7 +335,14 @@ class PrettyPrinter:
 
         for composite in composites:
             type_ = composite["__type__"]
-            if type_ in ("metadata", "validation", "connectionoptions"):
+            if type_ in (
+                "metadata",
+                "validation",
+                "connectionoptions",
+                "env",
+                "maps",
+                "plugins",
+            ):
                 # types are being parsed directly, and not as an attr of a parent
                 lines += self.process_key_dict(type_, composite, level=0)
             else:
@@ -354,7 +373,9 @@ class PrettyPrinter:
 
     def check_options_list(self, options_list, value):
         for option in options_list:
-            if "enum" in option and value.lower() in option["enum"]:
+            if ("enum" in option and value.lower() in option["enum"]) or (
+                "const" in option and value.lower() in option["const"]
+            ):
                 if value.lower() == "end":
                     # in GEOTRANSFORM "end" is an attribute value
                     return self.quoter.add_quotes(value)
@@ -389,6 +410,12 @@ class PrettyPrinter:
 
             return value
 
+        if "type" in attr_props and attr_props["type"] == "null":
+            # for keywords without a value return an empty string
+            # currently only CLASSAUTO is the only keyword without a value
+            assert attr.lower() == "classauto"
+            return ""
+
         if (
             "type" in attr_props and attr_props["type"] == "string"
         ):  # and "enum" not in attr_props
@@ -510,7 +537,9 @@ class PrettyPrinter:
 
         if isinstance(composite, dict) and "__type__" in composite:
             type_ = composite["__type__"]
-            assert type_ in COMPOSITE_NAMES.union(SINGLETON_COMPOSITE_NAMES)
+            assert type_ in COMPOSITE_NAMES.union(SINGLETON_COMPOSITE_NAMES).union(
+                CONFIG_COMPOSITE_NAMES
+            )
             is_hidden = False
             self._add_type_comment(level, comments, lines)
             s = self.whitespace(level, 0) + type_.upper()
@@ -534,10 +563,21 @@ class PrettyPrinter:
                     lines += self._format(v, level + 1)
             elif attr == "pattern":
                 lines += self.format_pair_list(attr, value, level)
-            elif attr in ("metadata", "validation", "values", "connectionoptions"):
+            elif attr in (
+                "metadata",
+                "validation",
+                "values",
+                "connectionoptions",
+            ):
                 # metadata and values are also composites
                 # but will be processed here
                 lines += self.process_key_dict(attr, value, level)
+            elif attr in (
+                "env",
+                "maps",
+                "plugins",
+            ):
+                lines += self.process_key_dict(attr, value, level, quote_key=False)
 
             elif attr == "projection":
                 projection_comments = self.process_attribute_comment(comments, attr)
