@@ -136,7 +136,7 @@ class MapfileTransformer(Transformer):
         self,
         composite_dict: dict,
         attribute_dict: dict,
-        position_dict: (dict | None),
+        position_dict: dict | None,
         pos: int,
     ):
         key_name = "config"
@@ -162,7 +162,7 @@ class MapfileTransformer(Transformer):
         self,
         composite_dict: dict,
         attribute_dict: dict,
-        position_dict: (dict | None),
+        position_dict: dict | None,
         pos: int,
     ):
         key_name = "points"
@@ -255,13 +255,13 @@ class MapfileTransformer(Transformer):
 
                     composite_dict[key_name].append(d[key_name])
 
-                    if position_dict:
+                    if position_dict is not None:
                         if key_name not in position_dict:
                             position_dict[key_name] = []
                         position_dict[key_name].append(pos)
                 else:
                     assert len(d.items()) == 1
-                    if position_dict:
+                    if position_dict is not None:
                         # hoist position details to composite
                         position_dict[key_name] = pos
                     if self.include_comments and comments:
@@ -329,7 +329,9 @@ class MapfileTransformer(Transformer):
 
         return d
 
-    def check_composite_tokens(self, name: str, tokens) -> tuple[str, list[Any]]:
+    def check_composite_tokens(
+        self, name: str, tokens: list[Any]
+    ) -> tuple[str, list[Any]]:
         """
         Return the key and contents of a KEY..END block
         for PATTERN, POINTS, and PROJECTION
@@ -640,6 +642,15 @@ class MapfileTransformer(Transformer):
         v.value = "{%s}" % list_values
         return v
 
+    def classauto(self, t):
+        key_token = t[0]
+        key_name = self.key_name(key_token)
+        pd = self.create_position_dict(key_token, None)
+        d: dict = OrderedDict()
+        d["__position__"] = pd
+        d[key_name] = None
+        return d
+
 
 class CommentsTransformer(Transformer_InPlace):
     """
@@ -722,10 +733,70 @@ class CommentsTransformer(Transformer_InPlace):
 
     # below we assign callbacks to process comments for each of the following types
     attr = _save_attr_comments
+    config_attr = _save_attr_comments  # handled the same as other attr comments
     projection = _save_projection_comments
     composite = _save_composite_comments
 
 
+class ConfigfileTransformer(MapfileTransformer):
+    def config(self, tree):
+        composite_dict = CaseInsensitiveOrderedDict(CaseInsensitiveOrderedDict)
+        composite_dict["__type__"] = "config"
+
+        for t in tree:
+            key = t.data.lower()
+
+            atts_dict: OrderedDict[str, Any] = OrderedDict()
+
+            if self.include_comments:
+                comments_dict = atts_dict["__comments__"] = OrderedDict()
+
+            if self.include_position:
+                # self.create_position_dict(key_name, None)
+                position_dict = atts_dict["__position__"] = OrderedDict()
+
+            for c in t.children:
+                # first remove dicts that are no longer required
+                pos = c.pop("__position__")
+                c.pop(
+                    "__tokens__", None
+                )  # tokens are no longer needed now we have the positions
+                comments = c.pop("__comments__", None)
+
+                #  simple attribute
+                assert len(c.items()) == 1
+                att = list(c.items())[0]
+                att_key = att[0]
+                att_value = att[1]
+
+                if self.include_position:
+                    # hoist position details to composite
+                    position_dict[att_key] = pos
+                if self.include_comments and comments:
+                    # hoist comments to composite
+                    comments_dict[att_key] = comments
+
+                if att_key in atts_dict.keys():
+                    log.warning(
+                        "A duplicate key (%s) was found in %s. Only the last value (%s) will be used. ",
+                        att_key,
+                        key,
+                        att_value,
+                    )
+                atts_dict[att_key] = att_value
+
+            composite_dict[key] = atts_dict
+
+        return composite_dict
+
+    def config_attr(self, tokens) -> dict:
+        """
+        Process CONFIG file attributes which can be quoted
+        or unquoted, otherwise they are identical to other attributes
+        """
+        return self.attr(tokens)
+
+
 class MapfileToDict:
     def __init__(
         self,
